/***************************************************/
/* Game Excelleration Kit vX                       */
/* Copyright (C) 2001 by HourGlass Software NonInc */
/***************************************************/
#ifndef __GEKMatrix__
#define __GEKMatrix__
/*! 
	\author MoJo
    \file GEKMatrix.h
    \brief 4x4 Matrix Class
*/

//! 4x4 Matrix Class
struct GEKMATRIX
{
public:
	union
	{
		float d[16];
		float m[4][4];

		struct
		{
			float _11, _12, _13, _14;	
			float _21, _22, _23, _24;
			float _31, _32, _33, _34;
			float _41, _42, _43, _44;
		};

		struct
		{
			GEKVECTOR4 rx;
			GEKVECTOR4 ry;
			GEKVECTOR4 rz;

			union
			{
				struct
				{
					GEKVECTOR3 t;
					float w;
				};

				struct
				{
					float x, y, z, w;
				};
			};
		};
	};

public:
	inline GEKMATRIX(void);
	inline GEKMATRIX(float *pfData);
	inline GEKMATRIX(float _11, float _12, float _13, float _14,
					 float _21, float _22, float _23, float _24,
					 float _31, float _32, float _33, float _34,
					 float _41, float _42, float _43, float _44);
	inline GEKMATRIX(const GEKMATRIX &kMatrix);
	inline GEKMATRIX(const GEKVECTOR3 &kAxis, float fAngle);
	inline GEKMATRIX(const GEKVECTOR3 &kEuler);

	//@{ 
	//! Assignment Operators
	inline GEKMATRIX operator = (const GEKMATRIX &kMatrix);
	void operator *= (const GEKMATRIX &kMatrix);
	//@}

	//@{ 
	//! Casting Operators
	inline float operator () (int iRow, int iCol) const;
	inline float operator () (int iRow, int iCol);
	inline operator const float * (void) const;
	inline operator float * (void);
	//@}

	//! Binary Operator
	GEKMATRIX operator * (const GEKMATRIX &kMatrix);

	inline void SetIdentity(void);								//!< Sets a matrix to identity
	void SetEuler(const GEKVECTOR3 &kEuler);					//!< Sets the XYZ euler angles of a rotation matrix
	void SetRotation(const GEKVECTOR3 &kAxis, float fAngle);	//!< Sets the axial angle of a rotation matrix
	void SetQuaternion(const GEKQUATERNION &kQuat);				//!< Converts a quaternion to a rotation matrix

	inline void SetScaling(const GEKVECTOR3 &kScale);			//!< Sets the scaling factor of a matrix
	inline GEKVECTOR3 GetScaling(void);							//!< Gets the scaling factor of a matrix

	void Transpose(void);										//!< Transposes a matrix
	inline float Determinant(void);								//!< Calculates the determinant of a matrix
	GEKMATRIX Inverse(void);									//!< Calculates the inverse of a matrix
	void Invert(void);											//!< Inverts a matrix

	//! Rotates a matrix preserving translations
	void Rotate(const GEKMATRIX &kMatrix);

	//@{
	//! Rotates a point by a rotation matrix
	GEKVECTOR3 Rotate(const GEKVECTOR2 &kVector);
	GEKVECTOR3 Rotate(const GEKVECTOR3 &kVector);
	GEKVECTOR3 Rotate(const GEKVECTOR4 &kVector);
	GEKVECTOR3 Rotate(const GEKVECTOR2 &kVector) const;
	GEKVECTOR3 Rotate(const GEKVECTOR3 &kVector) const;
	GEKVECTOR3 Rotate(const GEKVECTOR4 &kVector) const;
	//@}

	//@{
	//! Transforms a point by a matrix
	GEKVECTOR4 Transform(const GEKVECTOR2 &kVector);
	GEKVECTOR4 Transform(const GEKVECTOR3 &kVector);
	GEKVECTOR4 Transform(const GEKVECTOR4 &kVector);
	GEKVECTOR4 Transform(const GEKVECTOR2 &kVector) const;
	GEKVECTOR4 Transform(const GEKVECTOR3 &kVector) const;
	GEKVECTOR4 Transform(const GEKVECTOR4 &kVector) const;
	//@}
};

/*!
	\brief Creates an orthographic projection matrix
	\param fXMin [in] the minimum X value of the view volume
	\param fYMin [in] the minimum Y value of the view volume
	\param fXMax [in] the maximum X value of the view volume
	\param fYMax [in] the maximum Y value of the view volume
	\param fNear [in] the distance from the viewer to the near clipping plane
	\param fFar [in] the distance from the viewer to the far clipping plane

	The orthographic matrix is defined as follows:
<pre>
	fX		 0		  0		 fTX
	 0		fY		  0		 fTY
	 0		 0		-fZ		-fTZ
	 0		 0		  0		   1

	fX = (2.0f / (fXMax - fXMin))
	fY = (2.0f / (fYMax - fYMin))
	fZ = (-2.0f / (fFar - fNear))
	fTX = -((fXMax + fXMin) / (fXMax - fXMin))
	fTY = -((fYMax + fYMin) / (fYMax - fYMin))
	fTZ = -((fFar + fNear) / (fFar - fNear))
</pre>
*/
GEKMATRIX GEKMatrixOrtho(float fXMin, float fYMin, float fXMax, float fYMax, float fNear, float fFar);

/*!
	\brief Creates a perspective projection matrix
	\param fFOV [in] the field of view angle of the view volume
	\param fAspect [in] the width to height aspect ratio of the view volume
	\param fNear [in] the distance from the viewer to the near clipping plane
	\param fFar [in] the distance from the viewer to the far clipping plane
	
	The perspective matrix is defined as follows:
<pre>
	fX		 0		fA		 0
	 0		fY		fB		 0
	 0		 0		fC		fD
	 0		 0		-1		 0

	fX = ((2.0f * fNear) / (fXMax - fXMin))
	fY = ((2.0f * fNear) / (fYMax - fYMin))
	fA = ((fXMax + fXMin) / (fXMax - fXMin))
	fB = ((fYMax + fYMin) / (fYMax - fYMin))
	fC = -((fFar + fNear) / (fFar - fNear))
	fD = -((2.0f * fFar * fNear) / (fFar - fNear))
</pre>
*/
GEKMATRIX GEKMatrixPerspective(float fFOV, float fAspect, float fNear, float fFar);

/*!
	\brief Creates a matrix to mirror coordinates around a plane
	\param kPlane [in] the plane to mirror coordinates by
*/
GEKMATRIX GEKMatrixMirror(const GEKPLANE &kPlane);

/*!
	\brief Creates a view matrix based upon a direction
	\param kOrigin [in] the origin of the view matrix
	\param kTarget [in] the target of the view matrix
*/
GEKMATRIX GEKMatrixTarget(const GEKVECTOR3 &kOrigin, const GEKVECTOR3 &kTarget);

#endif